local function config()
local dap = require('dap')
dap.adapters.python = function(cb, config)
  if config.request == 'attach' then
    ---@diagnostic disable-next-line: undefined-field
    local port = (config.connect or config).port
    ---@diagnostic disable-next-line: undefined-field
    local host = (config.connect or config).host or '127.0.0.1'
    cb({
      type = 'server',
      port = assert(port, '`connect.port` is required for a python `attach` configuration'),
      host = host,
      options = {
        source_filetype = 'python',
      },
    })
  else
    cb({
      type = 'executable',
      command = '/Users/hedy/.config/nvim/.virtualenvs/debugpy/bin/python',
      args = { '-m', 'debugpy.adapter' },
      options = {
        source_filetype = 'python',
      },
    })
  end
end

dap.configurations.python = {
  {
    -- The first three options are required by nvim-dap
    type = 'python'; -- the type here established the link to the adapter definition: `dap.adapters.python`
    request = 'launch';
    name = "Launch file";

    -- Options below are for debugpy, see https://github.com/microsoft/debugpy/wiki/Debug-configuration-settings for supported options

    program = "${file}"; -- This configuration will launch the current file if used.
    pythonPath = function()
      -- debugpy supports launching an application with a different interpreter then the one used to launch debugpy itself.
      -- The code below looks for a `venv` or `.venv` folder in the current directly and uses the python within.
      -- You could adapt this - to for example use the `VIRTUAL_ENV` environment variable.
      local cwd = vim.fn.getcwd()
      if vim.fn.executable(cwd .. '/venv/bin/python') == 1 then
        return cwd .. '/venv/bin/python'
      elseif vim.fn.executable(cwd .. '/.venv/bin/python') == 1 then
        return cwd .. '/.venv/bin/python'
      else
        return '/usr/bin/python3'
      end
    end;
  },
}

require("nvim-dap-virtual-text").setup({})

local dapui = require("dapui")
dapui.setup({})

dap.listeners.after.event_initialized["dapui_config"] = function()
  dapui.open()
end
dap.listeners.before.event_terminated["dapui_config"] = function()
  dapui.close()
end
dap.listeners.before.event_exited["dapui_config"] = function()
  dapui.close()
end

vim.api.nvim_create_user_command(
  'DuiToggle',
  function(opts)
    local a = nil
    if opts and #opts.args > 0 then
      a = opts.args[1]
    end
    require("dapui").toggle(a)
  end,
  {
    nargs = '?',
    complete = function() return {'sidebar', 'tray'} end,
  })
end

return {
  -- TODO: check nvim version
  { "mfussenegger/nvim-dap",
    config = config,
    lazy = true,
    cmd = {"DapToggleBreakpoint", "DapContinue"},
  },
  { "theHamsta/nvim-dap-virtual-text",
    lazy = true,
    cmd = {"DapToggleBreakpoint", "DapContinue"},
    dependencies = {"mfussenegger/nvim-dap"},
  },
  { "rcarriga/nvim-dap-ui",
    lazy = true,
    cmd = {"DapToggleBreakpoint", "DapContinue"},
    dependencies = {"mfussenegger/nvim-dap"},
  },
}
